module kindlytypes
!--------------------------------------------------------------------------------
!
! Set up the kinds of integers, reals, etc needed for a single program.
!
! Beware: the actual integer value returned by selected_real_kind() is dependent
! on the compiler. So do NOT assume that real8 = 8. On SGI platforms, the returned
! integer value of real8 is 2. Also, the convention of declaring double precision 
! numbers via real*4 or real*8 is NOT a Fortran standard. Most compilers will let 
! you use those, but do not rely on it. 
!
! On 10 Jun 2012 added in the quad precision declarations based on the file
!   http://ftp.aset.psu.edu/pub/ger/fortran/hdk/QorDPrecision.f90
! The declaration in that file includes
!   Public domain 2004 by James Van Buskirk
! so give him credit, not me.
!
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!--------------------------------------------------------------------------------

    implicit none
    public 
    
    integer, parameter :: real4    = selected_real_kind(6,37)
    integer, parameter :: real8    = selected_real_kind(13,300)
    integer, parameter :: real16   = selected_real_kind(30,1000)
    integer, parameter :: integer4 = selected_int_kind(9)
    integer, parameter :: integer8 = selected_int_kind(18)
    !--------------------------------------------------------------------------
    ! selected_real_kind() gives a negative integer if the specs cannot be
    ! satisfied:
    !   -1 if precision is ok but exponent range is not achievable
    !   -2 if precision is not achievable but range is ok
    !   -3 if neither precision and range are not achievable 
    !   -4 a type exists for precision and a different type exists for the
    !      exponent range
    ! The sign() function returns +1 if real16 is nonnegative, -1 otherwise.
    ! So the trick here is that one of the two terms must be 0, depending only
    ! on the sign of real16 but not its particular value. 
    !--------------------------------------------------------------------------
    integer, parameter :: fullprecision = (1+sign(1, real16))/2*real16 + &
                                          (1-sign(1, real16))/2*real8
    integer, parameter :: wp = real8

end module kindlytypes
